#
# spec file for package chrony
#
# Copyright (c) 2016 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.
#
# Please submit bugfixes or comments via http://bugzilla.suse.com/
#

%define vendorzone opensuse.
%if 0%{?suse_version} > 1230
%bcond_without  systemd
%bcond_without  syscallfilter
%else
%bcond_with     systemd
%bcond_with     syscallfilter
%endif
%global clknetsim_ver 96416d5

Name:           chrony
Version:        2.3
Release:        6.1
License:        GPL-2.0
Summary:        System Clock Synchronization Client and Server
Group:          Productivity/Networking/Other
Url:            http://chrony.tuxfamily.org/
Source:         http://download.tuxfamily.org/chrony/chrony-%{version}.tar.gz
Source1:        chrony.init
Source2:        chrony.sysconfig
Source3:        chrony.dhclient
Source4:        chrony.helper
Source5:        chrony-dnssrv@.service
Source6:        chrony-dnssrv@.timer
# Simulator for test suite
Source10:       https://github.com/mlichvar/clknetsim/archive/%{clknetsim_ver}/clknetsim-%{clknetsim_ver}.tar.gz

# PATCH-MISSING-TAG -- See http://wiki.opensuse.org/openSUSE:Packaging_Patches_guidelines
Patch0:         chrony-config.patch
# Add NTP servers from DHCP when starting service
Patch1:         chrony-service-helper.patch
Patch2:         chrony-logrotate.patch
Patch3:         chrony-include-termios.patch
Patch4:         make-105-ntpauth-more-reliable.patch
Patch5:         chrony-fix-open.patch
#
BuildRequires:  NetworkManager-devel
BuildRequires:  bison
BuildRequires:  libcap-devel
BuildRequires:  libedit-devel
BuildRequires:  mozilla-nss-devel
BuildRequires:  pkg-config
BuildRequires:  texinfo
BuildRequires:  gcc-c++
# The timezone package is needed for the "make check" tests. It can be
# removed if the call to make check is ever deleted.
BuildRequires:  timezone

%if %{with syscallfilter}
%ifarch s390 s390x ppc64le
BuildRequires:  libseccomp-devel >= 2.2.0
%else
BuildRequires:  libseccomp-devel
%endif
%endif
%if %{with systemd}
BuildRequires:  pkgconfig(systemd)
%{?systemd_requires}
%else
Requires(pre):  %insserv_prereq
%endif
Requires:       logrotate
Requires(pre):  %fillup_prereq
Requires(pre):  %{_sbindir}/groupadd
Requires(pre):  %{_sbindir}/useradd
Requires(pre):  %{install_info_prereq}
Provides:       ntp-daemon
BuildRoot:      %{_tmppath}/%{name}-%{version}-build

%description
Chrony is an implementation of the Network Time Protocol (NTP). It can
synchronize the system clock with NTP servers, reference clocks (e.g. a
GPS receiver), and manual input using wristwatch and keyboard. It can
also operate as an NTPv4 (RFC 5905) server and peer to provide a time
service to other computers in the network.

Chrony consists of two programs: chronyd and chronyc.

Chronyd is a daemon which runs in the background on the system. It
obtains measurements of the system clock’s offset relative to time
servers on other systems via the network and adjusts the system time
accordingly. For isolated systems, the user can periodically enter the
correct time by hand (using chronyc). In either case, chronyd
determines the rate at which the computer gains or loses time, and
compensates for this. Chronyd can act as either a client or a server.

Chronyc provides a user interface to chronyd for monitoring its
performance and configuring various settings. It can do so while
running on the same computer as the chronyd instance it is controlling
or a different computer.

%prep
%setup -q -a 10
%patch0 -p1
%patch1 -p1
%patch2 -p1
%patch3 -p1
%patch4 -p1
%patch5 -p1

# use our vendor zone (2.*pool.ntp.org names include IPv6 addresses)
sed -e 's|^\(pool \)\(pool.ntp.org\)|\12.%{vendorzone}\2|' \
        < examples/chrony.conf.example2 > chrony.conf

touch -r examples/chrony.conf.example2 chrony.conf

# regenerate the file from getdate.y
rm -f getdate.c

mv clknetsim-%{clknetsim_ver}* test/simulation/clknetsim

%build
# not autoconf:
export CFLAGS="%{optflags} -Wall -fpic -DPIC $(pkg-config --cflags libseccomp)"
export LDFLAGS="-pie -Wl,-z,relro,-z,now"
%configure                                  \
  --docdir="%{_docdir}/%{name}"             \
  %if %{with syscallfilter}
  --enable-scfilter                         \
  %endif
  --with-user=chrony                        \
  --with-hwclockfile=%{_sysconfdir}/adjtime \
  --with-sendmail=%{_sbindir}/sendmail

make %{?_smp_mflags} all docs
gzip -9 -f -n chrony.txt

%install
make %{?_smp_mflags} DESTDIR=%{buildroot} install install-docs
rm "%{buildroot}%{_docdir}/%{name}/%{name}.html"

mkdir -p %{buildroot}%{_sysconfdir}/{sysconfig,logrotate.d}
mkdir -p %{buildroot}%{_localstatedir}/{lib,log}/chrony
mkdir -p %{buildroot}%{_sysconfdir}/NetworkManager/dispatcher.d
mkdir -p %{buildroot}%{_sysconfdir}/dhcp/dhclient.d
mkdir -p %{buildroot}%{_datadir}
mkdir -p %{buildroot}{%{_unitdir},%{_prefix}/lib/systemd/ntp-units.d}

install -m 644 -p chrony.conf %{buildroot}%{_sysconfdir}/chrony.conf

install -m 640 -p examples/chrony.keys.example \
        %{buildroot}%{_sysconfdir}/chrony.keys
install -m 755 -p examples/chrony.nm-dispatcher \
        %{buildroot}%{_sysconfdir}/NetworkManager/dispatcher.d/20-chrony
install -m 755 -p %{SOURCE3} \
        %{buildroot}%{_sysconfdir}/dhcp/dhclient.d/chrony.sh
install -m 644 -p examples/chrony.logrotate \
        %{buildroot}%{_sysconfdir}/logrotate.d/chrony

%if %{with systemd}
install -m 644 -p examples/chronyd.service \
        %{buildroot}%{_unitdir}/chronyd.service
install -m 644 -p examples/chrony-wait.service \
        %{buildroot}%{_unitdir}/chrony-wait.service
install -m 644 -p %{SOURCE5} %{buildroot}%{_unitdir}/chrony-dnssrv@.service
install -m 644 -p %{SOURCE6} %{buildroot}%{_unitdir}/chrony-dnssrv@.timer
ln -s /usr/sbin/service "%{buildroot}%{_sbindir}/rcchronyd"
ln -s /usr/sbin/service "%{buildroot}%{_sbindir}/rcchrony-wait"

echo 'chronyd.service' > \
        %{buildroot}%{_prefix}/lib/systemd/ntp-units.d/50-chronyd.list
%else
install -D -m0755 "%{SOURCE1}" "%{buildroot}%{_initddir}/chronyd"
ln -s ../..%{_initddir}/chronyd "%{buildroot}%{_sbindir}/rcchronyd"
%endif
install -D -m0644 "%{SOURCE2}" "%{buildroot}%{_localstatedir}/adm/fillup-templates/sysconfig.chronyd"

install -m 755 -p %{SOURCE4} %{buildroot}%{_datadir}/chrony-helper

touch %{buildroot}%{_localstatedir}/lib/chrony/{drift,rtc}

%check
# Set random seed to get deterministic results
export CLKNETSIM_RANDOM_SEED=24501
make %{?_smp_mflags} -C test/simulation/clknetsim

make check

%pre
%{_sbindir}/groupadd -r chrony >/dev/null 2>&1 || :
%{_sbindir}/useradd -g chrony -s /bin/false -r -c "Chrony Daemon" -d "%{_localstatedir}/lib/chrony" chrony >/dev/null 2>&1 || :
# START BIG SYSTEMD
%if %{with systemd}
%service_add_pre chronyd.service chrony-wait.service
%endif

%preun
%if %{with systemd}
%service_del_preun chronyd.service chrony-wait.service
%else
%stop_on_removal chronyd
%endif

%post
%fillup_only -n chronyd
%install_info --info-dir="%{_infodir}" "%{_infodir}/chrony.info%{ext_info}"
%if %{with systemd}
%service_add_post chronyd.service chrony-wait.service
%endif

%postun
%install_info_delete --info-dir="%{_infodir}" "%{_infodir}/chrony.info%{ext_info}"
%if %{with systemd}
%service_del_postun chronyd.service chrony-wait.service
%else
%restart_on_update chronyd
%insserv_cleanup
%endif

%files
%defattr(-,root,root)
%doc FAQ NEWS README chrony.txt* COPYING
%doc examples
%config(noreplace) %attr(0640,root,%{name}) %{_sysconfdir}/chrony.conf
%config(noreplace) %attr(0640,root,%{name}) %verify(not md5 size mtime) %{_sysconfdir}/chrony.keys
%config(noreplace) %{_sysconfdir}/logrotate.d/chrony
%{_sysconfdir}/NetworkManager/dispatcher.d/20-chrony
%dir %{_sysconfdir}/dhcp/
%dir %{_sysconfdir}/dhcp/dhclient.d/
%{_sysconfdir}/dhcp/dhclient.d/chrony.sh
%{_bindir}/chronyc
%{_sbindir}/chronyd
%{_datadir}/chrony-helper
%{_mandir}/man1/chronyc.1%{ext_man}
%{_mandir}/man5/chrony.conf.5%{ext_man}
%{_mandir}/man8/chronyd.8%{ext_man}
%{_infodir}/chrony.info*
%if %{with systemd}
%{_prefix}/lib/systemd/ntp-units.d/*.list
%{_unitdir}/chrony*.service
%{_unitdir}/chrony*.timer
%else
%config %{_sysconfdir}/init.d/chronyd
%endif
%{_sbindir}/rcchrony*
%{_localstatedir}/adm/fillup-templates/sysconfig.chronyd
%dir %attr(750,chrony,chrony) %{_localstatedir}/lib/chrony
%ghost %attr(640,chrony,chrony) %{_localstatedir}/lib/chrony/drift
%ghost %attr(640,chrony,chrony) %{_localstatedir}/lib/chrony/rtc
%dir %attr(750,chrony,chrony) %{_localstatedir}/log/chrony

%changelog
* Fri Dec 14 2018 Martin Pluskal <mpluskal@suse.com>
- Make sure to generate correct sysconfig file (boo#1117147)
* Wed Apr 13 2016 mpluskal@suse.com
- Provide ntp-daemon (bsc#973981)
* Mon Apr 11 2016 meissner@suse.com
- chrony-fix-open.patch: make sure _open and _close are initialized
  in open()/close() override, as libfreebl3 also calls from the
  the ELF constructor. FATE#319508
- enable mozilla-nss
* Fri Apr  8 2016 mpluskal@suse.com
- Use correct license
- Drop hardcoded dependency on libseccomp, it is detected during
  build
* Fri Apr  8 2016 mpluskal@suse.com
- Undo reference to chrony-dnssrv@.service in %%pre, %%preun, %%post,
  and %%postun as it would lead to error.
- Change conditions for libseccom, we can use any version on SLE-12
  x86_64
* Tue Apr  5 2016 mpost@suse.com
- Removed %%if for distributions that aren't building chrony.
- Renamed chrony-2.2_logrotate.patch to chrony-logrotate.patch since
  the patch is not particularly version-dependent.
- Added clknetsim for "make check" processing.
- Added Buildrequires for gcc-c++ and timezone for building clknetsim
  and running "make check".
- Changed Buildrequires and Requires to specify the minimum level of
  libseccomp needed to build on s390x and ppc64le.
- Removed "-Recommends: timedatex" since I couldn't find any instance
  of it anywhere in the build service.
- Modified the description to use some of the information from the
  chrony web site.
- Added chrony-include-termios.patch so that it will build on ppc64le.
- Added make-105-ntpauth-more-reliable.patch so that "make check"
  will not report a non-failure as a failure.
- Added --without-nss to ./configure to avoid "interruption code
  0x2003B in chronyd" errors.
- Changed the symbolic links for rcchronyd and rcchronyd-wait to
  point to the actual location of the service command, not the symlink
  in /sbin.
- Added reference to chrony-dnssrv@.service in %%pre, %%preun, %%post,
  and %%postun.
* Mon Mar 28 2016 mpluskal@suse.com
- Cleanup spec file with spec-cleaner
- Prepare for submission to Factory (see fate#319508)
* Thu Feb 18 2016 mrueckert@suse.de
- update to 2.3
  - Enhancements
  - Add support for NTP and command response rate limiting
  - Add support for dropping root privileges on Mac OS X,
    FreeBSD, Solaris
  - Add require and trust options for source selection
  - Enable logchange by default (1 second threshold)
  - Set RTC on Mac OS X with rtcsync directive
  - Allow binding to NTP port after dropping root privileges on
    NetBSD
  - Drop CAP_NET_BIND_SERVICE capability on Linux when NTP port
    is disabled
  - Resolve names in separate process when seccomp filter is
    enabled
  - Replace old records in client log when memory limit is
    reached
  - Don't reveal local time and synchronisation state in client
    packets
  - Don't keep client sockets open for longer than necessary
  - Ignore poll in KoD RATE packets as ntpd doesn't always set it
    correctly
  - Warn when using keys shorter than 80 bits
  - Add keygen command to generate random keys easily
  - Add serverstats command to report NTP and command packet
    statistics
  - Bug fixes
  - Fix clock correction after making step on Mac OS X
  - Fix building on Solaris
- refreshed patches to apply cleanly again:
  chrony-2.2_logrotate.patch
  chrony-config.patch
  chrony-service-helper.patch
* Fri Jan 29 2016 mrueckert@suse.de
- update to 2.2.1
  Restrict authentication of NTP server/peer to specified key
  (CVE-2016-1567)
* Thu Nov 26 2015 mrueckert@suse.de
- silence groupadd/useradd call and drop the shell from the user.
* Thu Nov 26 2015 mrueckert@suse.de
- update to 2.2
  see /usr/share/doc/packages/chrony/NEWS
- sync with fedora spec and add systemd support
- refreshed chrony-config.patch to apply cleanly again
- added chrony-2.2_logrotate.patch: add missing su option as we no
  longer have the daemon run as root.
- added chrony-service-helper.patch: imported from fedora with a
  changed path for moving from libexecdir to datadir
- only use syscall filters on 12.3 and newer
- move helper from libexecdir to datadir
* Mon Feb 24 2014 mrueckert@suse.de
- clean up build section
  - the configure script can actually import CC/CFLAGS from the
    environment. no need to break any CFLAGS it might set in the
    configure script.
  - remove unneeded prefix from the make calls.
  - enable building the binaries with PIE/relro now
* Mon Feb 24 2014 mrueckert@suse.de
- Update to version 1.29.1:
  * Modify chronyc protocol to prevent amplification attacks
    (CVE-2014-0021) (incompatible with previous protocol version,
    chronyc supports both)
- Additional changes from 1.29
  * Fix crash when processing crafted commands (CVE-2012-4502)
    (possible with IP addresses allowed by cmdallow and localhost)
  * Don't send uninitialized data in SUBNETS_ACCESSED and
    CLIENT_ACCESSES replies (CVE-2012-4503) (not used by chronyc)
  * Drop support for SUBNETS_ACCESSED and CLIENT_ACCESSES commands
- Additional changes from 1.28
  * Combine sources to improve accuracy
  * Make config and command parser strict
  * Add -a option to chronyc to authenticate automatically
  * Add -R option to ignore initstepslew and makestep directives
  * Add generatecommandkey, minsamples, maxsamples and user
    directives
  * Improve compatibility with NTPv1 and NTPv2 clients
  * Create sockets only in selected family with -4/-6 option
  * Treat address bind errors as non-fatal
  * Extend tracking log
  * Accept float values as initstepslew threshold
  * Allow hostnames in offline, online and burst commands
  * Fix and improve peer polling
  * Fix crash in config parsing with too many servers
  * Fix crash with duplicated initstepslew address
  * Fix delta calculation with extreme frequency offsets
  * Set local stratum correctly
  * Remove unnecessary adjtimex calls
  * Set paths in documentation by configure
  * Update chrony.spec
- Updated chrony-config.patch:
  - lots of config values were fixed upstream already
  - key file patching is unnecessary
* Sat Jul 13 2013 zaitor@opensuse.org
- Update to version 1.27:
  + Added support for stronger authentication keys via NSS or
    libtomcrypt library.
  + Extended tracking, sources and activity reports printed by
    chronyc.
  + The daemon now waits in foreground until it is fully
    initialized.
  + Other bug fixes and improvements.
- Add mozilla-nss-devel & pkg-config BuildRequires, new optional
  dependencys.
* Fri Jan 11 2013 mrdocs@opensuse.org
-run spec-cleaner on the spec file, fix license and remove cruft
* Tue Nov 29 2011 aj@suse.de
- Update to version 1.26:
  * Added compatibility with Linux 3.0 and later
  * Fixed replying on multihomed IPv6 hosts
  * Other minor bug fixes and improvements
- Cleanup package a bit.
